/*
 * Copyright (c) 2000, Amnon BARAK (amnon@cs.huji.ac.il). All rights reserved.
 *
 *       MOSIX $Id: libmosix.h,v 1.10 2001/03/06 01:31:04 amnons Exp $
 *
 * Permission to use, copy and distribute this software is hereby granted 
 * under the terms of version 2 or any later version of the GNU General Public
 * License, as published by the Free Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED IN ITS "AS IS" CONDITION, WITH NO WARRANTY
 * WHATSOEVER. NO LIABILITY OF ANY KIND FOR ANY DAMAGES WHATSOEVER RESULTING
 * FROM THE USE OF THIS SOFTWARE WILL BE ACCEPTED.
 */
/*
 * Author(s): Ariel Rosenblatt
 * Code derived from previous work by Amnon Shiloh and Oren Laadan.
 */
#ifndef __libmosix__
#define __libmosix__

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <ctype.h>
#include <sys/socket.h>
#include <linux/mosctl.h>
#include <linux/mosix.h>
/*#include <mos/defs.h>*/
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/utsname.h>

#define MAX_PATH_LENGTH              256

/*#define OVERHEADS_NUMBER             (MAX_MOSIX_TOPOLOGY * sizeof(struct mosix_cost) / sizeof(int)) */

#define proc(dir)                    "/proc/" #dir "/"
#define proc_mosix(dir)              proc(mosix) #dir "/"

/* /proc/mosix/nodes */
#define msx_nodes(i)                 proc_mosix(nodes) #i "/"
#define msx_nodes_item_i(i, item)    msx_nodes(i) #item

/* /proc/self */
#define msx_self(item)               proc(self) #item

/* /proc/mosix/info */
#define msx_info_item(item)          proc_mosix(info) #item

/* /proc/mosix/decay */
#define msx_decay_item(item)         proc_mosix(decay) #item

/* /proc/mosix/admin */
#define msx_admin_item(item)         proc_mosix(admin) #item

/* /proc/self/entries */
#define MSX_SELF_WHERE_FN           msx_self(where)
#define MSX_SELF_MIGRATE_FN         msx_self(migrate)
#define MSX_SELF_GOTO_FN            msx_self(goto)
#define MSX_SELF_LOCK_FN            msx_self(lock)
#define MSX_SELF_CANTMOVE_FN        msx_self(cantmove)
#define MSX_SELF_NMIGS_FN           msx_self(nmigs)

/* /proc/mosix/info entries */
#define MSX_INFO_LOADS_FN           msx_info_item(loads)
#define MSX_INFO_SPEEDS_FN          msx_info_item(speeds)
#define MSX_INFO_CPUS_FN            msx_info_item(cpus)
#define MSX_INFO_MEMS_FN            msx_info_item(mems)
#define MSX_INFO_RMEMS_FN           msx_info_item(rmems)
#define MSX_INFO_TMEMS_FN           msx_info_item(tmems)
#define MSX_INFO_UTILS_FN           msx_info_item(utils)
#define MSX_INFO_STATA_FN           msx_info_item(stata)
#define MSX_INFO_INFOS_FN           msx_info_item(infos)

/* /proc/mosix/decay entries */
#define MSX_DECAY_CPUJOB_FN         msx_decay_item(cpujob)
#define MSX_DECAY_IOJOB_FN          msx_decay_item(iojob)
#define MSX_DECAY_SLOW_FN           msx_decay_item(slow)
#define MSX_DECAY_FAST_FN           msx_decay_item(fast)
#define MSX_DECAY_OWN_FN            msx_decay_item(own)
#define MSX_DECAY_INHERIT_FN        msx_decay_item(inherit)
#define MSX_DECAY_EXEC_FN           msx_decay_item(exec)
#define MSX_DECAY_EXECONCE_FN       msx_decay_item(execonce)
#define MSX_DECAY_CLEAR_FN          msx_decay_item(clear)

/* /proc/mosix/admin entries */
#define MSX_ADMIN_CONFIG_FN         msx_admin_item(config)
#define MSX_ADMIN_STAY_FN           msx_admin_item(stay)
#define MSX_ADMIN_LSTAY_FN          msx_admin_item(lstay)
#define MSX_ADMIN_BLOCK_FN          msx_admin_item(block)
#define MSX_ADMIN_QUIET_FN          msx_admin_item(quiet)
#define MSX_ADMIN_NOMFS_FN          msx_admin_item(nomfs)
#define MSX_ADMIN_EXPEL_FN          msx_admin_item(expel)
#define MSX_ADMIN_BRING_FN          msx_admin_item(bring)
#define MSX_ADMIN_SPEED_FN          msx_admin_item(speed)
#define MSX_ADMIN_SSPEED_FN         msx_admin_item(sspeed)
#define MSX_ADMIN_OVERHEADS_FN      msx_admin_item(overheads)
#define MSX_ADMIN_SLOWDECAY_FN      msx_admin_item(slowdecay)
#define MSX_ADMIN_FASTDECAY_FN      msx_admin_item(fastdecay)
#define MSX_ADMIN_DECAYINTERVAL_FN  msx_admin_item(decayinterval)
#define MSX_ADMIN_MOSPE_FN          msx_admin_item(mospe)
#define MSX_ADMIN_TUNE_FN           msx_admin_item(tune)

#define msx_query_type(msx_query)         (0xFF00 & (msx_query))
#define msx_query_subtype(msx_query)      (0x00FF & (msx_query))


/* /proc/nnnnn entries */
char* msx_p_fn(pid_t pid, char *file);
char* msx_p_where_fn(pid_t pid);
char* msx_p_migrate_fn(pid_t pid);
char* msx_p_goto_fn(pid_t pid);
char* msx_p_lock_fn(pid_t pid);
char* msx_p_cantmove_fn(pid_t pid);
char* msx_p_nmigs_fn(pid_t pid);

/* /proc/mosix/nodes/n/ entries */
char* msx_node_fn(int node, char *file);
char* msx_node_load_fn(int node);  
char* msx_node_speed_fn(int node); 
char* msx_node_cpus_fn(int node);  
char* msx_node_mem_fn(int node);   
char* msx_node_rmem_fn(int node);  
char* msx_node_tmem_fn(int node); 
char* msx_node_util_fn(int node); 
char* msx_node_status_fn(int node);

enum msx_queries {
	MSX_P                 =  0x0000,
	MSX_P_WHERE           =  0x0001,
	MSX_P_MIGRATE         =  0x0002,
	MSX_P_GOTO            =  0x0003,
	MSX_P_LOCK            =  0x0004,
	MSX_P_CANTMOVE        =  0x0005,
	MSX_P_NMIGS           =  0x0006,

	MSX_NODE              =  0x0100,
	MSX_NODE_LOAD         =  0x0101,
	MSX_NODE_SPEED        =  0x0102,
	MSX_NODE_CPUS         =  0x0103,
	MSX_NODE_MEM          =  0x0104,
	MSX_NODE_RMEM         =  0x0105,
	MSX_NODE_TMEM         =  0x0106,
	MSX_NODE_UTIL         =  0x0107,
	MSX_NODE_STATUS       =  0x0108,

	MSX_INFO              =  0x0200,
	MSX_INFO_LOADS        =  0x0201,
	MSX_INFO_SPEEDS       =  0x0202,
	MSX_INFO_CPUS         =  0x0203,
	MSX_INFO_MEMS         =  0x0204,
	MSX_INFO_RMEMS        =  0x0205,
	MSX_INFO_TMEMS        =  0x0206,
	MSX_INFO_UTILS        =  0x0207,
	MSX_INFO_STATA        =  0x0208,
	MSX_INFO_INFOS        =  0x0209,

	MSX_DECAY             =  0x0300,
	MSX_DECAY_CPUJOB      =  0x0301,
	MSX_DECAY_IOJOB       =  0x0302,
	MSX_DECAY_SLOW        =  0x0303,
	MSX_DECAY_FAST        =  0x0304,
	MSX_DECAY_OWN         =  0x0305,
	MSX_DECAY_INHERIT     =  0x0306,
	MSX_DECAY_EXEC        =  0x0307,
	MSX_DECAY_EXECONCE    =  0x0308,
	MSX_DECAY_CLEAR       =  0x0309,

	MSX_ADMIN             =  0x0400,
	MSX_ADMIN_CONFIG      =  0x0401,
	MSX_ADMIN_STAY        =  0x0402,
	MSX_ADMIN_LSTAY       =  0x0403,
	MSX_ADMIN_BLOCK       =  0x0404,
	MSX_ADMIN_QUIET       =  0x0405,
	MSX_ADMIN_EXPEL       =  0x0406,
	MSX_ADMIN_BRING       =  0x0407,
	MSX_ADMIN_SPEED       =  0x0408,
	MSX_ADMIN_SSPEED      =  0x0409,
	MSX_ADMIN_OVERHEADS   =  0x040A,
	MSX_ADMIN_SLOWDECAY   =  0x040B,
	MSX_ADMIN_FASTDECAY   =  0x040C,
	MSX_ADMIN_DECAYINTERVAL= 0x040D,
	MSX_ADMIN_MOSPE       =  0x040E,
	MSX_ADMIN_TUNE        =  0x040F,
	MSX_ADMIN_NOMFS	      =  0x0410,

	MSX_SELF              =  0x0500,
	MSX_SELF_WHERE        =  0x0501,
	MSX_SELF_MIGRATE      =  0x0502,
	MSX_SELF_GOTO         =  0x0503,
	MSX_SELF_LOCK         =  0x0504,
	MSX_SELF_CANTMOVE     =  0x0505,
	MSX_SELF_NMIGS        =  0x0506
};

typedef enum msx_queries msx_query_t;
int proc_msx_get(msx_query_t query, int node_or_pid, void *resp, int len);
#define _proc_msx_get(query, n, d) (proc_msx_get((query), (n), (d), sizeof(*(d))))
int proc_msx_put(msx_query_t query, int node_or_pid, void *buff, int len);
#define _proc_msx_put(query, n, d) (proc_msx_put((query), (n), (d), sizeof(*(d))))
int proc_msx_exchange_int(msx_query_t query, int node_or_pid, int newval);

struct decay_params {
	int interval; 
	int slow; 
	int fast;
};

struct process_info {
	pid_t pid;
	int where;
};

enum msx_cmd { 
	D_STAY,      /* Disable automatic migration from here */
	D_NOSTAY,    /* Allow automatic migrations from here */
	D_LSTAY,     /* Disable automatic mig. of local processes */
	D_NOLSTAY,   /* Allow automatic mig.  of local processes */
	D_BLOCK,     /* Block automatic migration to here */
	D_NOBLOCK,   /* Enable automatic migration to here */
	D_EXPEL,     /* Expel all processes to remote processors */
	D_BRING,     /* Bring back all processes */
	D_GETLOAD,   /* Get current load */
	D_QUIET,     /* Stop internal load-balancing activity */
	D_NOQUIET,   /* Resume internal load-balancing activity */
	D_TUNE,      /* Enter tuning mode */
	D_NOTUNE,    /* Exit tuning mode */
	D_NOMFS,     /* Disallow MFS access to this node */
	D_MFS,       /* Reallow MFS access to this node */
	D_SETSSPEED,/* Set the standard speed, affecting D_GETLOAD */
	D_GETSSPEED, /* Get the standard speed (default=1000) */
	D_GETSPEED,  /* Get machine's speed */
	D_SETSPEED,  /* Set machine's speed */
	D_MOSIX_TO_IP,    /* Convert MOSIX to IP address */
	D_IP_TO_MOSIX,    /* Convert IP to MOSIX address */
	D_HOWMANY_MIGS,   /* return number of migrations for the process */
	D_GETTUNE,        /* get kernel tuning parameters */
	D_SETTUNE,        /* set kernel tuning parameters */
	D_GETSTAT,        /* get MOSIX status */
	D_GETMEM,         /* get current memory (free and total) */
	D_GETDECAY,       /* get decay parameters */
	D_SETDECAY,       /* set decay parameters */
	D_GETBMEM,        /* get OS's idea of memory (free and total) */
	D_GETUTIL,        /* get CPU utilizability % */
	D_SETWHERETO,     /* send a process somewhere */
};

#define D_GOBACKHOME    0       /* just go back home */
#define D_BALANCE       (-1)    /* perform load balancing */
#define D_LEAVE         (-2)    /* leave the process where it is */

#define D_UNLOCK        0
#define D_LOCK          1
#define D_NOCHANGE      2
#define D_MIGNOLOCK     3
	
typedef enum msx_cmd msx_cmd_t;
int msxctl(msx_cmd_t cmd, int arg, void *resp, int len);
#define msxctl1(x) (msxctl((x), 0, NULL, 0))
#define msxctl2(x,y) (msxctl((x), (y), NULL, 0))
#define msxctl3(x,y,z) (msxctl((x), (y), (z), sizeof(*(z))))

int proc_msx_read(char *path, void *buff, int len, off_t offset);
int proc_msx_write(char *path, void *buff, int len);

int msx_admin_read(msx_query_t query, void *resp, int len);
int msx_admin_write(msx_query_t query, void *arg, int len);

int msx_decay_read(msx_query_t query, void *resp);
int msx_decay_write(msx_query_t query, void *arg);

int msx_info_read(msx_query_t query, int node, void *buff, int len);

int msx_node_read(msx_query_t query, int node);

int msx_p_read(msx_query_t query, pid_t pid, void *resp, int len);
int msx_p_write(msx_query_t query, pid_t pid, void *arg);

int msx_self_read(msx_query_t query, void *resp, int len);
int msx_self_write(msx_query_t query,  void *arg);
#endif
